/*
        SC16IS750 SPI-to-UART-Chip on In-Circuit radino WiFi modules
        for more information: www.in-circuit.de or www.radino.cc

	Copyright (c) 2014 In-Circuit GmbH

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the "Software"), to deal in the Software without restriction,
	including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
	LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*
 * Simple implementation of basic functions for
 * communication over SPI-UART, similar to 'Serial' library
 *
 * -> print(String)
 * -> println(String)
 * -> print(int)
 * -> println(int)
 * -> available()
 * -> txavailable()
 * -> read()
 * -> write(char)
 *
 * Refer to SC16IS750.h for pin and register definitions
*/

#include "Arduino.h"
#include "SPI_UART.h"

// Use SPI-library for communication with SC16IS750
#include <SPI.h>

// Callback pointer for interrupt
static cb_type_name interruptCallbackPointer = NULL;



// Init SC16IS750
uint8_t SPI_UART::init()
{
	pinMode(_chipselect, OUTPUT);	// Set Chip select as output
	digitalWrite(_chipselect,HIGH);	//Clear Chip Select
	
	// start the SPI library:
	SPI.begin();  
	//Data is transmitted and received MSB first
	SPI.setBitOrder(MSBFIRST);  
	//SPI interface will run at 1MHz if 8MHz chip or 2Mhz if 16Mhz
	SPI.setClockDivider(SPI_CLOCK_DIV8);   
	//Data is clocked on the rising edge and clock is low when inactive
	SPI.setDataMode(SPI_MODE0);
  
	// Setup UART: 8N1
	// 8 databits, no parity, 1 stop-bit
	uint8_t uartSetting = (0<<LCR_DIVISOR) | (0<<LCR_BREAK) | (0<<LCR_PARITY_FORCED) | (0<<LCR_PARITY_TYPE) | (0<<LCR_PARITY_EN) | (0<<LCR_STOP) | (3<<LCR_WORD_LENGTH);  // 8N1
  
	///Line Control Register:
	// Save UART settings
	writeRegister(LCR,uartSetting);
  
	//Modem Control Register
	//Normal Operating Mode
	writeRegister(MCR,0x00);

	//FIFO Control Register: Clear & Enable the FIFO and no other features
	flush();

	return 1;
}

// Enable sleep mode globally when idle
void SPI_UART::enableSleeping(void)
{
	uint8_t buff = 0;
	
	buff = readRegister(LCR);  // Save last state of LCR
	writeRegister(LCR, 0xBF);  // Set to 0xBF to enable enhanced register set
	
	uint8_t buff2 = readRegister(EFR);
	
	buff2 |= (1<<ENABLE_ENHANCED_FUNCTIONS);  // enable enhanced functions
	
	writeRegister(EFR, buff2); // Save settings
	
	writeRegister(LCR, buff);  // Write old register value back
	
	buff  =  0;
	buff  =  readRegister(IER);  // get value
	buff |= (1<<SLEEP_MODE);  // Enable sleeping
	writeRegister(IER, buff);  // Save new settings
}

// Clear RX & TX-buffer and enable FIFO
void SPI_UART::flush()
{
  //FIFO Control Register: Clear & Enable the FIFO and no other features
  writeRegister(FCR,0x07);
}

void SPI_UART::print(const String dataString)
{
  print(dataString.c_str());
}

void SPI_UART::println(const String dataString)
{
  println(dataString.c_str());
}

// Print a string over UART
void SPI_UART::print(const char * dataString)
{
  const unsigned char * pStr;
//  Serial1.print(dataString);
  for (pStr=(const unsigned char *)dataString ; *pStr!='\0' ; pStr++)
  {
    write(*pStr);
  }

//  uint8_t i;
//  for(i=0;i<dataString.length();i++)
//  {
//    SC16_write(dataString[i]);
//  }
}

// Print a string over UART and append CR & LF
void SPI_UART::println(const char * dataString)
{
  print(dataString);
  write(13);  // CR - Carriage Return
  write(10);  // LF - Line Feed
}

// Print a number over UART as a string
void SPI_UART::print(uint16_t dat)
{
  char number[10];  
  print(itoa(dat, number, 10));
}

// Print a number over UART as a string and append CR & LF
void SPI_UART::println(uint16_t dat)
{
  print(dat);
  write(13);
  write(10);
}

// Get number of available bytes of the UART
// that wait to be received
uint8_t SPI_UART::available(void)
{
  return readRegister(RXLVL);
}

// Get number of available bytes within the
// UART's TX-Buffer. If data is sent while this returns 0,
// all sent data will be lost and not transmitted, because
// the buffer is already full.
uint8_t SPI_UART::txavailable(void)
{
  return readRegister(TXLVL);
}

// Read one Byte from UART
uint8_t SPI_UART::read(void)
{
  return readRegister(RHR);
}

// Write one Byte to UART
void SPI_UART::write(uint8_t data)
{
  while(!txavailable()) delay(1);//wait until TX-buffer is ready to receive at least one character
//  Serial1.write(data);
  writeRegister(THR,data); 
}

// Set GPIO-Pin (0..7) direction/mode
// 1 = Output == SC16_GPIO_INPUT
// 0 = Input == SC16_GPIO_OUTPUT
void SPI_UART::GPIO_pinMode(uint8_t pin, uint8_t mode)
{
  if(pin>7) return;  // wrong pin number, return
  uint8_t state = readRegister(IODir); // Get current pin setup
  
  // Only modify the corresponding bit of the desired pin
  if(mode == SC16_GPIO_OUTPUT){
    state |= (1<<pin);  // Set Output
  }else {
    state &= ~(1<<pin);  // Set Input
  }
  writeRegister(IODir, state);  // Save new pin setup
}

// Set value of GPIO pin (0..7)
// 0 = LOW
// 1 = HIGH
void SPI_UART::GPIO_digitalWrite(uint8_t pin, uint8_t value)
{
  if(pin>7) return;  // wrong pin number, return
  uint8_t state = readRegister(IOState); // Get current pin setup
  
  // Only modify the corresponding bit of the desired pin
  if(value){
    state |= (1<<pin);  // Set High
  }else {
    state &= ~(1<<pin);  // Set Low
  }
  writeRegister(IOState, state);  // Save new pin value
}

// Read value of GPIO pin (0..7)
// 0 = LOW
// 1 = HIGH
uint8_t SPI_UART::GPIO_digitalRead(uint8_t pin)
{
  if(pin>7) return 0;  // wrong pin number, return
  uint8_t state = readRegister(IOState); // Get current pin setup
  
  state &= 1<<pin;  // Get corresponding bit
  
  if(state) return 1;
    else return 0;  
}

// ATmega INT 6 Interrupt routine
ISR(INT6_vect) {
	SPI_UART::runCallback(0);
}

// Execute callback function
void SPI_UART::runCallback(uint8_t val)
{
	interruptCallbackPointer(val);
}

// Enable RXlvl Interrupt
uint8_t SPI_UART::defineInterrupts(uint8_t type)
{
	if(type != 0)
	{
		// Attach Interrupt 6
		pinMode(7, INPUT);	// INT6 as input
		// disable interrupt while changing flags
		EIMSK |= (0<<INT6);
		// trigger interrupt in case of low level of INT6
		EICRB = (0<<ISC61)|(0<<ISC60);
		EIMSK |= (1<<INT6); // Set bit 6 to 1 -> Enable INT6
	}else{
		EIMSK &= ~(1<<INT6); // Set bit 6 to 1 -> Disable INT6
	}
	
	writeRegister(IER, type);	// Enable/Disable Interrupts on SC16

	return 1;
}

// Attach function which should be called after each interrupt
void SPI_UART::attachInterruptCallback(cb_type_name function)
{
	interruptCallbackPointer = function;
	
}

// Set Baudrate of SC16IS750
void SPI_UART::setBaudrate(uint32_t baudrate)
{
  uint8_t help = 0x00;
  
  uint32_t divisor = 0;
  
  // e.g. Baudrate: 19200; divisor = (14.7456Mhz/ (16*19200) ) = 48
  divisor = (uint32_t)14745600 / 16;
  divisor = divisor / baudrate;
    
  help = readRegister(LCR);
  help |= (1<<7);  // Set Bit7 to 1 to enable special register set
  writeRegister(LCR,help);
  
  // DLL & DLH are part of the special register set
  writeRegister(DLL,divisor & 0xFF);  // Set Baudrate divisor, Low byte
  writeRegister(DLH,divisor >>8);  // High byte
  
  help &= ~(1<<7); // Set Bit7 to 0 to disable special register set
  writeRegister(LCR,help);
}

// Writes a value to a specified register over SPI
void SPI_UART::writeRegister(uint8_t thisRegister, uint8_t thisValue) {
  // SC16IS740 expects a R/W  bit first, followed by the 4 bit
  // register address of the byte.
  // So shift the bits left by three bits:
  thisRegister = thisRegister << 3;
  // now combine the register address and the command into one byte:
  uint8_t dataToSend = thisRegister | WRITE;

  // take the chip select low to select the device:
  digitalWrite(_chipselect, LOW);

  SPI.transfer(dataToSend); //Send register location
  SPI.transfer(thisValue);  //Send value to record into register

  // take the chip select high to de-select:
  digitalWrite(_chipselect, HIGH);

}

// Reads a value from a specified register over SPI
byte SPI_UART::readRegister(uint8_t thisRegister) {
  uint8_t readback;
  // SC16IS740 expects a R/W  bit first, followed by the 4 bit
  // register address of the byte.
  // So shift the bits left by three bits:
  thisRegister = thisRegister << 3;
  // now combine the register address and the command into one byte:
  uint8_t dataToSend = thisRegister | READ;

  // take the chip select low to select the device:
  digitalWrite(_chipselect, LOW);
  
  SPI.transfer(dataToSend); //Send register location
  readback = SPI.transfer(0);  //Get Value from register
  
  digitalWrite(_chipselect, HIGH);
  return(readback);
} 


